@@ -84,7 +84,8 @@ def l2_info(root_ids, progress=True, max_threads=4, dataset='production'):
     # Get/Initialize the CAVE client
     client = get_cave_client(dataset)
 
-    l2_ids = client.chunkedgraph.get_leaves(root_ids, stop_layer=2)
+    get_l2_ids = partial(retry(client.chunkedgraph.get_leaves), stop_layer=2)
+    l2_ids = get_l2_ids(root_ids)
 
     attributes = ['area_nm2', 'size_nm3', 'max_dt_nm', 'rep_coord_nm']
     info = client.l2cache.get_l2data(l2_ids.tolist(), attributes=attributes)
@@ -167,16 +168,24 @@ def l2_graph(root_ids, progress=True, dataset='production'):
     # This is a (N,2) array of edges
     l2_eg = np.array(client.chunkedgraph.level2_chunk_graph(root_ids))
 
-    # Drop duplicate edges
-    l2_eg = np.unique(np.sort(l2_eg, axis=1), axis=0)
-
+    # Generate graph
     G = nx.Graph()
-    G.add_edges_from(l2_eg)
+
+    if not len(l2_eg):
+        # If no edges, this neuron consists of a single chunk
+        # Get the single chunk's ID
+        chunks = client.chunkedgraph.get_leaves(root_ids, stop_layer=2)
+        G.add_nodes_from(chunks)
+    else:
+        # Drop duplicate edges
+        l2_eg = np.unique(np.sort(l2_eg, axis=1), axis=0)
+
+        G.add_edges_from(l2_eg)
 
     return G
 
 
-def l2_skeleton(root_id, refine=True, drop_missing=True,
+def l2_skeleton(root_id, refine=True, drop_missing=True, omit_failures=None,
                 progress=True, dataset='production', **kwargs):
     """Generate skeleton from L2 graph.
 
@@ -192,7 +201,14 @@ def l2_skeleton(root_id, refine=True, drop_missing=True,
                         Only relevant if ``refine=True``: if True, will drop
                         nodes that don't have a corresponding chunk mesh. These
                         are typically chunks that are either very small or very
-                        new.
+                        new. Note that this might result in empty neurons.
+    omit_failures :     bool, optional
+                        Determine behaviour when skeleton generation fails
+                        (e.g. if the neuron has only a single chunk):
+                         - ``None`` (default) will raise an exception
+                         - ``True`` will skip the offending neuron (might result
+                           in an empty ``NeuronList``)
+                         - ``False`` will return an empty ``TreeNeuron``
     progress :          bool
                         Whether to show a progress bar.
     **kwargs
@@ -201,8 +217,8 @@ def l2_skeleton(root_id, refine=True, drop_missing=True,
 
     Returns
     -------
-    skeleton :          navis.TreeNeuron
-                        The extracted skeleton.
+    skeleton(s) :       navis.TreeNeuron | navis.NeuronList
+                        The extracted L2 skeleton.
 
     Examples
     --------
@@ -210,28 +226,50 @@ def l2_skeleton(root_id, refine=True, drop_missing=True,
     >>> n = flywire.l2_skeleton(720575940614131061)
 
     """
+    if omit_failures not in (None, True, False):
+        raise ValueError('`omit_failures` must be either None, True or False. '
+                         f'Got "{omit_failures}".')
     # TODO:
     # - drop duplicate nodes in unrefined skeleton
     # - use L2 graph to find soma: highest degree is typically the soma
 
     if navis.utils.is_iterable(root_id):
         nl = []
-        for id in navis.config.tqdm(root_id, desc='Skeletonizing',
+        for id in navis.config.tqdm(root_id, desc='L2 skeletons',
                                     disable=not progress, leave=False):
             n = l2_skeleton(id, refine=refine, drop_missing=drop_missing,
                             progress=progress, dataset=dataset, **kwargs)
             nl.append(n)
         return navis.NeuronList(nl)
 
+    # Turn into integer
+    root_id = int(root_id)
+
     # Get the cloudvolume
     vol = parse_volume(dataset)
 
     # Get/Initialize the CAVE client
     client = get_cave_client(dataset)
 
-    # Load the L2 graph for given root ID
-    # This is a (N,2) array of edges
-    l2_eg = np.array(client.chunkedgraph.level2_chunk_graph(root_id))
+    # Load the L2 graph for given root ID (this is a (N, 2) array of edges)
+    get_l2_edges = retry(client.chunkedgraph.level2_chunk_graph)
+    l2_eg = get_l2_edges(root_id)
+
+    # If no edges, we can't create a skeleton
+    if not len(l2_eg):
+        msg = (f'Unable to create L2 skeleton: root ID {root_id} '
+               'consists of only a single L2 chunk.')
+        if omit_failures == None:
+            raise ValueError(msg)
+
+        navis.config.logger.warning(msg)
+
+        if omit_failures:
+            # If omission simply return an empty NeuronList
+            return navis.NeuronList([])
+            # If no omission, return empty TreeNeuron
+        else:
+            return navis.TreeNeuron(None, id=root_id, units='1 nm', **kwargs)
 
     # Drop duplicate edges
     l2_eg = np.unique(np.sort(l2_eg, axis=1), axis=0)
@@ -276,15 +314,31 @@ def l2_skeleton(root_id, refine=True, drop_missing=True,
 
         # Map refined coordinates onto the SWC
         has_new = swc.node_id.isin(new_co)
-        swc.loc[has_new, 'x'] = swc.loc[has_new, 'node_id'].map(lambda x: new_co[x][0])
-        swc.loc[has_new, 'y'] = swc.loc[has_new, 'node_id'].map(lambda x: new_co[x][1])
-        swc.loc[has_new, 'z'] = swc.loc[has_new, 'node_id'].map(lambda x: new_co[x][2])
+
+        # Only apply if we actually have new coordinates - otherwise there
+        # the datatype is changed to object for some reason...
+        if any(has_new):
+            swc.loc[has_new, 'x'] = swc.loc[has_new, 'node_id'].map(lambda x: new_co[x][0])
+            swc.loc[has_new, 'y'] = swc.loc[has_new, 'node_id'].map(lambda x: new_co[x][1])
+            swc.loc[has_new, 'z'] = swc.loc[has_new, 'node_id'].map(lambda x: new_co[x][2])
 
         # Turn into a proper neuron
         tn = navis.TreeNeuron(swc, id=root_id, units='1 nm', **kwargs)
 
         # Drop nodes that are still at their unrefined chunk position
         if drop_missing:
+            if not any(has_new):
+                msg = (f'Unable to refine: no L2 info for root ID {root_id} '
+                       'available. Set `drop_missing=False` to use unrefined '
+                       'positions.')
+                if omit_failures == None:
+                    raise ValueError(msg)
+                elif omit_failures:
+                    return navis.NeuronList([])
+                    # If no omission, return empty TreeNeuron
+                else:
+                    return navis.TreeNeuron(None, id=root_id, units='1 nm', **kwargs)
+
             tn = navis.remove_nodes(tn, swc.loc[~has_new, 'node_id'].values)
     else:
         tn = navis.TreeNeuron(swc, id=root_id, units='1 nm', **kwargs)
@@ -292,10 +346,12 @@ def l2_skeleton(root_id, refine=True, drop_missing=True,
     return tn
 
 
-def l2_dotprops(root_ids, min_size=None, progress=True, max_threads=10,
-                dataset='production', **kwargs):
+def l2_dotprops(root_ids, min_size=None, omit_failures=None, progress=True,
+                max_threads=10, dataset='production', **kwargs):
     """Generate dotprops from L2 chunks.
 
+    L2 chunks not present in the L2 cache are silently ignored.
+
     Parameters
     ----------
     root_ids  :         int | list of ints
@@ -306,7 +362,14 @@ def l2_dotprops(root_ids, min_size=None, progress=True, max_threads=10,
                         will be ignored. This is useful to de-emphasise the
                         finer terminal neurites which typically break into more,
                         smaller chunks and are hence overrepresented. A good
-                        value appears to be around 1,000,000.
+                        value appears to be around 1_000_000.
+    omit_failures :     bool, optional
+                        Determine behaviour when dotprops generation fails
+                        (i.e. if the neuron has no L2 info):
+                         - ``None`` (default) will raise an exception
+                         - ``True`` will skip the offending neuron (might result
+                           in an empty ``NeuronList``)
+                         - ``False`` will return an empty ``Dotprops``
     progress :          bool
                         Whether to show a progress bar.
     max_threads :       int
@@ -327,6 +390,10 @@ def l2_dotprops(root_ids, min_size=None, progress=True, max_threads=10,
     >>> n = flywire.l2_dotprops(720575940614131061)
 
     """
+    if omit_failures not in (None, True, False):
+        raise ValueError('`omit_failures` must be either None, True or False. '
+                         f'Got "{omit_failures}".')
+
     if not navis.utils.is_iterable(root_ids):
         root_ids = [root_ids]
 
@@ -339,18 +406,17 @@ def l2_dotprops(root_ids, min_size=None, progress=True, max_threads=10,
     client = get_cave_client(dataset)
 
     # Load the L2 IDs
-    # Note that we are using the L2 graph endpoint as I have not yet found a
-    # faster way to query the IDs.
     with ThreadPoolExecutor(max_workers=max_threads) as pool:
-        futures = pool.map(retry(client.chunkedgraph.level2_chunk_graph), root_ids)
-        l2_eg = [f for f in navis.config.tqdm(futures,
-                                              desc='Fetching L2 IDs',
-                                              total=len(root_ids),
-                                              disable=not progress or len(root_ids) == 1,
-                                              leave=False)]
+        get_l2_ids = partial(retry(client.chunkedgraph.get_leaves), stop_layer=2)
+        futures = pool.map(get_l2_ids, root_ids)
+        l2_ids = [f for f in navis.config.tqdm(futures,
+                                               desc='Fetching L2 IDs',
+                                               total=len(root_ids),
+                                               disable=not progress or len(root_ids) == 1,
+                                               leave=False)]
 
-    # Unique L2 IDs per root ID
-    l2_ids = [np.unique(g).astype(str) for g in l2_eg]
+    # Turn IDs into strings
+    l2_ids = [i.astype(str) for i in l2_ids]
 
     # Flatten into a list of all L2 IDs
     l2_ids_all = np.unique([i for l in l2_ids for i in l])
@@ -373,21 +439,33 @@ def l2_dotprops(root_ids, min_size=None, progress=True, max_threads=10,
             pbar.update(len(chunk))
 
     # L2 chunks without info will show as empty dictionaries
-    # Let's drop them to make our life easier
-    l2_info_ids = [k for k, v in l2_info.items() if v]
+    # Let's drop them to make our life easier (speeds up indexing too)
+    l2_info = {k: v for k, v in l2_info.items() if v}
 
     # Generate dotprops
     dps = []
     for root, ids in navis.config.tqdm(zip(root_ids, l2_ids),
                                        desc='Creating dotprops',
                                        total=len(root_ids),
-                                       disable=not progress or len(root_ids) == 1,
+                                       disable=not progress or len(root_ids) <= 1,
                                        leave=False):
-        # Find out for which IDs we have info
-        ids = ids[np.isin(ids, l2_info_ids)]
-
         # Get xyz points and the first component of the PCA as vector
-        this_info = [l2_info[i] for i in ids]
+        # Note that first subsetting IDs to what's actually available in
+        # `l2_info` is actually slower than doing it like this
+        this_info = [l2_info[i] for i in ids if i in l2_info]
+
+        if not len(this_info):
+            msg = ('Unable to create L2 dotprops: none of the L2 chunks for '
+                   f'root ID {root} are present in the L2 cache.')
+            if omit_failures == None:
+                raise ValueError(msg)
+
+            if not omit_failures:
+                # If no omission, add empty Dotprops
+                dps.append(navis.Dotprops(None, k=None, id=root,
+                                          units='1 nm', **kwargs))
+            continue
+
         pts = np.vstack([i['rep_coord_nm'] for i in this_info])
         vec = np.vstack([i['pca'][0] for i in this_info])
 
